@@ -17,6 +17,8 @@
 #include "sizeclasstable.h"
 #include "slab.h"
 
+#include <functional>
+
 namespace snmalloc
 {
   enum Boundary
@@ -60,7 +62,10 @@ namespace snmalloc
    * passed the global allocator.  The second initialises the thread-local
    * allocator if it is has been been initialised already. Splitting into two
    * functions allows for the code to be structured into tail calls to improve
-   * codegen.
+   * codegen.  The second template takes a function that takes the allocator
+   * that is initialised, and the value returned, is returned by
+   * `InitThreadAllocator`.  This is used incase we are running during teardown
+   * and the thread local allocator cannot be kept alive.
    *
    * The `MemoryProvider` defines the source of memory for this allocator.
    * Allocators try to reuse address space by allocating from existing slabs or
@@ -78,7 +83,7 @@ namespace snmalloc
    */
   template<
     bool (*NeedsInitialisation)(void*),
-    void* (*InitThreadAllocator)(),
+    void* (*InitThreadAllocator)(function_ref<void*(void*)>),
     class MemoryProvider = GlobalVirtual,
     class ChunkMap = SNMALLOC_DEFAULT_CHUNKMAP,
     bool IsQueueInline = true>
@@ -106,7 +111,7 @@ namespace snmalloc
       return large_allocator.stats;
     }
 
-    template<class MP>
+    template<class MP, class Alloc>
     friend class AllocPool;
 
     /**
@@ -199,6 +204,35 @@ namespace snmalloc
 #endif
     }
 
+    /**
+     * Checks the allocation at `p` could have been validly allocated with
+     * a size of `size`.
+     */
+    void check_size(void* p, size_t size)
+    {
+#if defined(CHECK_CLIENT)
+      auto asize = alloc_size(p);
+      auto asc = size_to_sizeclass(asize);
+      if (size_to_sizeclass(size) != asc)
+      {
+        // Correction for large classes.
+        if (asc > NUM_SIZECLASSES)
+        {
+          if (bits::next_pow2(size) != asize)
+            error("Deallocating with incorrect size supplied.");
+        }
+        // Correction for zero sized allocations.
+        else if ((size != 0) && (asc != 0))
+        {
+          error("Deallocating with incorrect size supplied.");
+        }
+      }
+#else
+      UNUSED(p);
+      UNUSED(size);
+#endif
+    }
+
     /*
      * Free memory of a statically known size. Must be called with an
      * external pointer.
@@ -210,6 +244,7 @@ namespace snmalloc
       UNUSED(size);
       return free(p);
 #else
+      check_size(p, size);
       constexpr sizeclass_t sizeclass = size_to_sizeclass_const(size);
 
       if (sizeclass < NUM_SMALL_CLASSES)
@@ -249,6 +284,7 @@ namespace snmalloc
       UNUSED(size);
       return free(p);
 #else
+      check_size(p, size);
       if (likely((size - 1) <= (sizeclass_to_size(NUM_SMALL_CLASSES - 1) - 1)))
       {
         Superslab* super = Superslab::get(p);
@@ -1095,7 +1131,7 @@ namespace snmalloc
         stats().sizeclass_alloc(sizeclass);
         return small_alloc_new_free_list<zero_mem, allow_reserve>(sizeclass);
       }
-      return small_alloc_first_alloc<zero_mem, allow_reserve>(sizeclass, size);
+      return small_alloc_first_alloc<zero_mem, allow_reserve>(size);
     }
 
     /**
@@ -1103,12 +1139,11 @@ namespace snmalloc
      * then directs the allocation request to the newly created allocator.
      */
     template<ZeroMem zero_mem, AllowReserve allow_reserve>
-    SNMALLOC_SLOW_PATH void*
-    small_alloc_first_alloc(sizeclass_t sizeclass, size_t size)
+    SNMALLOC_SLOW_PATH void* small_alloc_first_alloc(size_t size)
     {
-      auto replacement = InitThreadAllocator();
-      return reinterpret_cast<Allocator*>(replacement)
-        ->template small_alloc_inner<zero_mem, allow_reserve>(sizeclass, size);
+      return InitThreadAllocator([size](void* alloc) {
+        return reinterpret_cast<Allocator*>(alloc)->alloc(size);
+      });
     }
 
     /**
@@ -1285,10 +1320,9 @@ namespace snmalloc
       {
         if (NeedsInitialisation(this))
         {
-          void* replacement = InitThreadAllocator();
-          return reinterpret_cast<Allocator*>(replacement)
-            ->template medium_alloc<zero_mem, allow_reserve>(
-              sizeclass, rsize, size);
+          return InitThreadAllocator([size](void* alloc) {
+            return reinterpret_cast<Allocator*>(alloc)->alloc(size);
+          });
         }
         slab = reinterpret_cast<Mediumslab*>(
           large_allocator.template alloc<NoZero, allow_reserve>(
@@ -1359,9 +1393,9 @@ namespace snmalloc
 
       if (NeedsInitialisation(this))
       {
-        void* replacement = InitThreadAllocator();
-        return reinterpret_cast<Allocator*>(replacement)
-          ->template large_alloc<zero_mem, allow_reserve>(size);
+        return InitThreadAllocator([size](void* alloc) {
+          return reinterpret_cast<Allocator*>(alloc)->alloc(size);
+        });
       }
 
       size_t size_bits = bits::next_pow2_bits(size);
@@ -1386,9 +1420,11 @@ namespace snmalloc
 
       if (NeedsInitialisation(this))
       {
-        void* replacement = InitThreadAllocator();
-        return reinterpret_cast<Allocator*>(replacement)
-          ->large_dealloc(p, size);
+        InitThreadAllocator([p](void* alloc) {
+          reinterpret_cast<Allocator*>(alloc)->dealloc(p);
+          return nullptr;
+        });
+        return;
       }
 
       size_t size_bits = bits::next_pow2_bits(size);
@@ -1439,10 +1475,10 @@ namespace snmalloc
       // a real allocator and construct one if we aren't.
       if (NeedsInitialisation(this))
       {
-        void* replacement = InitThreadAllocator();
-        // We have to do a dealloc, not a remote_dealloc here because this may
-        // have been allocated with the allocator that we've just had returned.
-        reinterpret_cast<Allocator*>(replacement)->dealloc(p);
+        InitThreadAllocator([p](void* alloc) {
+          reinterpret_cast<Allocator*>(alloc)->dealloc(p);
+          return nullptr;
+        });
         return;
       }
 
