@@ -73,7 +73,7 @@ void test_random_allocation()
         cell = alloc->alloc(16);
         auto pair = allocated.insert(cell);
         // Check not already allocated
-        assert(pair.second);
+        SNMALLOC_ASSERT(pair.second);
         UNUSED(pair);
         alloc_count++;
       }
@@ -130,14 +130,14 @@ void test_double_alloc()
     for (size_t i = 0; i < (n * 2); i++)
     {
       void* p = a1->alloc(20);
-      assert(set1.find(p) == set1.end());
+      SNMALLOC_ASSERT(set1.find(p) == set1.end());
       set1.insert(p);
     }
 
     for (size_t i = 0; i < (n * 2); i++)
     {
       void* p = a2->alloc(20);
-      assert(set2.find(p) == set2.end());
+      SNMALLOC_ASSERT(set2.find(p) == set2.end());
       set2.insert(p);
     }
 
@@ -178,8 +178,8 @@ void test_external_pointer()
       void* p4 = Alloc::external_pointer<End>(p2);
       UNUSED(p3);
       UNUSED(p4);
-      assert(p1 == p3);
-      assert((size_t)p4 == (size_t)p1 + size - 1);
+      SNMALLOC_ASSERT(p1 == p3);
+      SNMALLOC_ASSERT((size_t)p4 == (size_t)p1 + size - 1);
     }
 
     alloc->dealloc(p1, size);
@@ -281,7 +281,7 @@ void test_alloc_16M()
   const size_t size = 16'000'000;
 
   void* p1 = alloc->alloc(size);
-  assert(Alloc::alloc_size(Alloc::external_pointer(p1)) >= size);
+  SNMALLOC_ASSERT(Alloc::alloc_size(Alloc::external_pointer(p1)) >= size);
   alloc->dealloc(p1);
 }
 
@@ -292,7 +292,7 @@ void test_calloc_16M()
   const size_t size = 16'000'000;
 
   void* p1 = alloc->alloc<YesZero>(size);
-  assert(Alloc::alloc_size(Alloc::external_pointer(p1)) >= size);
+  SNMALLOC_ASSERT(Alloc::alloc_size(Alloc::external_pointer(p1)) >= size);
   alloc->dealloc(p1);
 }
 
